#include "assets.hpp"
#include "collision.hpp"
#include "texture.hpp"
#include <SDL_image.h>
#include <SDL_pixels.h>
#include <SDL_render.h>
#include <config.hpp>
#include <iostream>
#include <phoenix.hpp>
#include <state.hpp>

GlobalState state;

void PRunBoostrap() {
  PLoadTextureAsset("assets/textures/wall_brick.png");
  PLoadTextureAsset("assets/textures/floor.png");
  PLoadTextureAsset("assets/player/player_walk_down.png");
  PLoadTextureAsset("assets/player/player_walk_up.png");
  PLoadTextureAsset("assets/player/player_walk_left.png");
  PLoadTextureAsset("assets/player/player_walk_right.png");

  PPopulateCollisionDb();
}

PhoenixGame::PhoenixGame() {

  SDL_Init(SDL_INIT_VIDEO);
  IMG_Init(IMG_INIT_PNG);

  state.window = SDL_CreateWindow("Phoenix", SDL_WINDOWPOS_CENTERED,
                                  SDL_WINDOWPOS_CENTERED, 1000, 1000, 0);

  state.renderer = SDL_CreateRenderer(
      state.window, -1, SDL_RENDERER_PRESENTVSYNC | SDL_RENDERER_SOFTWARE);

  // Load textures
  PRunBoostrap();

  m_currentMap = PLoadMapFromFile("maps/ph_example.map");
  PBuildCollisionFromMap(&m_currentMap);

  state.camera.x = 0;
  state.camera.y = 0;
  state.camera.w = SCREEN_WIDTH;
  state.camera.h = SCREEN_HEIGHT;

  state.player.x = m_currentMap.mapPlayerSpawnX;
  state.player.y = m_currentMap.mapPlayerSpawnY;
}

void PhoenixGame::present() { SDL_RenderPresent(state.renderer); }

void PhoenixGame::render() {

  SDL_RenderSetScale(state.renderer, m_currentMap.mapScaleX,
                     m_currentMap.mapScaleY);
  SDL_SetRenderDrawColor(state.renderer, 0, 0, 0, 255);
  SDL_RenderClear(state.renderer);

  PRenderMap(&m_currentMap);
  PRenderPlayer(&state.player);

  //  PRenderTexture(&m_texture, 0, 0);
}

void PhoenixGame::run() {
  mb_isOpen = true;

  while (mb_isOpen) {
    SDL_Event event;

    while (SDL_PollEvent(&event)) {
      if (event.type == SDL_QUIT) {
        mb_isOpen = false;
        break;
      }
    }

    unsigned char const *keys = SDL_GetKeyboardState(nullptr);

    if (keys[SDL_SCANCODE_W] &&
        PCanMoveTo(state.player.x, state.player.y - 5)) {
      state.player.facing = Up;
      state.player.y -= 5;
    }

    if (keys[SDL_SCANCODE_S] &&
        PCanMoveTo(state.player.x, state.player.y + 5)) {
      state.player.facing = Down;
      state.player.y += 5;
    }

    if (keys[SDL_SCANCODE_A] &&
        PCanMoveTo(state.player.x - 5, state.player.y)) {
      state.player.facing = Left;
      state.player.x -= 5;
    }

    if (keys[SDL_SCANCODE_D] &&
        PCanMoveTo(state.player.x + 5, state.player.y)) {
      state.player.facing = Right;
      state.player.x += 5;
    }

    state.camera.x =
        state.player.x - ((SCREEN_WIDTH / m_currentMap.mapScaleX) / 2);
    state.camera.y =
        state.player.y - ((SCREEN_HEIGHT / m_currentMap.mapScaleY) / 2);

    // Rendering code

    render();
    present();

    // End rendering code
  }
}
